% Dynamical System of the Khan King and Wolman (2001) Competitive Equilibrium Model.
 
% housekeeping
clear
clc
close all
history = 30;  

% title
disp ( '		Dynamic Solution of the Khan King and Wolman model of Competitive Equilibrium  ' )
disp ( '  ' )

filename = input( ' steady state or complete filename (examples: base, small): ','s' );

save nowfile filename

cd Calibrations
eval ( [ ' load ', filename ] )
cd ..

if exist('M') ~= 1;	% model solution does not exist, use King and Watson method
	
	disp ( ' Model solution with King and Watson programs KWREV98_1 ' )
	checklocation = exist('resolkw');
    if ( checklocation ~= 2 )        
        disp ( ' cannot locate King and Watson directory KW98REV_1 ' )
        disp ( ' cannot manage things unless this directory is in path ' )        
    end
	
	% declare the linearisation and driving process file names
	LinearisationFile = 'DynamicLinearCE';
	DrivingProcessFile = 'DynamicDrv';
	tracksolution = 0;
	
   % execute the King and Watson solution program.
   
	[M,PI, xlist, ylist] = resolkw(LinearisationFile,DrivingProcessFile,tracksolution);
   
   % append results to the steady state file
   saveresults = input( 'save dynamic solution? (y/n) ','s');
   if (saveresults == 'y')
   	cd Calibrations
   	eval ( [ ' save ', filename, ' J Xsol daytime param shock M PI xlist ylist ' ] )
   	cd .. 
	end

   delete nowfile.mat
	
else
	
	disp ( ' Using existing model solution ' )
	
end

% Generate Impulse Reponses if required.

shocktype = input( ' shock to (a) productivity (g) government spending (z) money demand or exit (e)? ','s');

if (shocktype ~= 'e')
   
   shockway =  input( ' enter shock size: ');
   
   numstates = size(M);
   numstates = numstates(1);
      
   J = param(11);
   steadyinfl = Xsol(6+3*J);
   steadyinfl = 1/steadyinfl;
   steadyxi = Xsol(2+2*J);
   steadyc = Xsol(1+2*J);
        
   % Set up and shock the dynamical system 
   nvar = size(ylist); nvar = nvar(1,1);   
   S = zeros(numstates,history);
   
   if (shocktype == 'g')
      S(numstates - 1,1) = shockway; 
      shocktype = 'spending';
      is = 1;
   elseif (shocktype == 'z')
      S(numstates,1) = shockway;
      shocktype = 'moneydemand';
      is = 0;
   else
      S(numstates - 2,1) = shockway;
      shocktype = 'productivity';
      is = 2;
   end
   
   F(:,1) = PI*S(:,1);
   
   % record impulses
   for time = 2:1:history
      S(:,time) = M*S(:,time - 1);
      F(:,time) = PI*S(:,time);
	end      
   
   % Locate variables of interest   
   ic = find(ylist(:,1) == 'c');
   iR = find(ylist(:,1) == 'R'); iR = iR(1);
   iShare = find(ylist(:,1) == 's'); iShare = iShare(1:J);
   in = find(ylist(:,1) == 'n');
   ii = find(ylist(:,1) == 'i');
   ixi = find(ylist(:,1) == 'x');
   ilambda = 3 + 2*J;
   ileisure = 4 + 2*J;
   
   % convert inverse inflation into inflation
   inflation = F(ii,:);
   inflation = -1*inflation;
   
   consumption = F(ic,:);
   labour = F(in,:);
   nominalrate = F(iR,:);
   Shares = F(iShare,:);
   xi = F(ixi, :);
   leisure = F(ileisure, :);
   lambda = F(ilambda, :);
   wage = -leisure - lambda;
   markup = S(numstates - 2,:) - wage;
   
   % generate price level deviations from trend
   zp = 1;				% zero index
   P(zp) = 1;
   Ptrend(zp) = 1;
   
   for i = 1:1:history
      infllevel(i) = inflation(i)*steadyinfl + steadyinfl;
      P(zp+ i) = P(zp + i-1)*infllevel(i);
      Ptrend(zp + i) = Ptrend(zp + i - 1)*steadyinfl;
      xilev(i) = xi(i)*steadyxi + steadyxi;
   end
   
   P = P(2:history+1);
   Ptrend = Ptrend(2:history + 1);
   Mtrend = Ptrend*(1 - steadyxi)*steadyc;
   Pdev = (P - Ptrend)./Ptrend;
   M = (1 - xilev).*consumption.*P;
   Mdev = (M - Mtrend)./Mtrend;
    
   infldev = infllevel - steadyinfl;   
   realrate = nominalrate - infldev; 
  
   nominalrate = nominalrate*4;
   realrate = realrate*4;
     
   % plot figures
   time = [1:1:history];
   
   figure
   
   subplot(311)
   
   title ([ shocktype ' shock process ' ] )
   hold on
   plot(time, S(numstates - is,:), 'm+')
      
   subplot (312)
   
   title ( ' inflation ' )
   hold on
   plot(time, inflation, 'r')
   
   subplot (313)
   
   colours = 'mcrgbky';
   
   title ( ' shares ')
   hold on
   
   for i = 1:1:J
      colour = i;
      colour = min(i, mod(i,length(colours))+ 1);
      colourchoice = colours(colour);
      plot(time, Shares(i,:), colourchoice)
      [yval, xval] = max(Shares(i,:));
      labelnow = ['s(' num2str(i) ')'];
      text(xval - 0.5, yval + 0.005, labelnow)
      hold on
   end
   
   figure
   
   subplot(221)
   
   title( ' consumption [r--] and labour [b-] ' )
   hold on
   plot( time, consumption, 'r--', time, labour, 'b')
     
   subplot(222)
   
   title ( ' real rate deviation ' )
   hold on
   plot (time, realrate)
      
   subplot(223)
   
   title( ' nominal rate deviation ' )
   hold on
   plot(time, nominalrate)
     
   subplot(224)
   
   title ( ' Prices % dev. trend ')
   hold on
   plot(time, Pdev)
   
   figure
   
   subplot(221)
   
   title ( ' \xi: share of credit goods ' )
   hold on
   plot(time, xi)
   
   subplot(222)
   title ( ' Money % dev. trend ' )
   hold on
   plot(time, Mdev)
   
   subplot(223)
   title ( ' wage rate ' )
   hold on
   plot(time, wage)
   
   subplot(224)
   title ( ' markup ' )
   hold on
   plot(time, markup)

end