% Dynamical System of the Khan King and Wolman (2000) 'Optimal Monetary Policy' Model.
% 
% 040600 initial program
%
% xi, money, wage and markup figures were added during 11 and 12 June 2000 
% 14.22.2000 extended to beta distribution
% modified 06.12.2001, at Bob's asking to see Pbar dynamics.
%
% Instructions for user: 
% 
% To solve a new model, first solve the steady state using steadydriver.  You must enter
% the parameters for the model and excellent initial conditions in STEADYDRIVER.M.  Next
% run steadydriver.  This will store the result param and Xsol, in a mat-file.  To examine
% these results, use recoversteady.m.
% 
% Next to solve the dynamical system, run Dynamicnew.M.  Enter the name of the mat-file you
% have generated which describes the economy.  You may verify variable locations using 
% steadydriver and adjust which series are displayed by dynamic. 
%
% Details of program families:
%
% Steady State solved in Steadydriver (SteadyState, SteadyNEQ, System0) -> steadystate.mat
% 
% Linearised dynamical system is generated by Dynamicnew (DynamicLinear (DynamicList), DynamicDRV).
%
% Dynamical System solved by King and Watson algor6 (KWREV98_1) from resolkw.m

% housekeeping
clear
clc
close all
history = 30;  

% title
disp ( '		Dynamic Solution of the Khan King and Wolman model of Optimal Monetary Policy ' )
disp ( '  ' )

filename = input( ' steady state or complete filename (examples: base, small): ','s' );

save nowfile filename

cd Calibrations
eval ( [ ' load ', filename ] )
cd ..

if exist('M') ~= 1;	% model solution does not exist, use King and Watson method
	
	disp ( ' Model solution with King and Watson programs KWREV98_1 ' )
	
   checklocation = exist('resolkw');
   if ( checklocation ~= 2 )        
        disp ( ' cannot locate King and Watson directory KW98REV_1 ' )
        disp ( ' cannot manage things unless this directory is in path ' )        
    end
	
	% declare the linearisation and driving process file names
	LinearisationFile = 'DynamicLinear';
	DrivingProcessFile = 'DynamicDrv';
	tracksolution = 0;
	
   % execute the King and Watson solution program.
   
	[M,PI, xlist, ylist] = resolkw(LinearisationFile,DrivingProcessFile,tracksolution);
   
   % append results to the steady state file
   saveresults = input( 'save dynamic solution? (y/n) ','s');
   if (saveresults == 'y')
   	cd Calibrations
   	eval ( [ ' save ', filename, ' J Xsol daytime param shock M PI xlist ylist ' ] )
   	cd .. 
	end

   delete nowfile.mat
	
else
	
	disp ( ' Using existing model solution ' )
	
end

% Generate Impulse Reponses if required.

shocktype = input( ' shock to (a) productivity (g) government spending (z) money demand or exit (e)? ','s');

if (shocktype ~= 'e')
   
   shockway =  input( ' enter shock size: ');
   
   numstates = size(M);
   numstates = numstates(1);
      
   J = param(11);
   steadynominalrate = Xsol(6+3*J);  
   steadyinfl = Xsol(12+3*J);
   steadyrealrate = (steadynominalrate + 1)*steadyinfl;
   steadyrealrate = steadyrealrate - 1;
   steadyinfl = 1/steadyinfl;
   steadyxi = Xsol(8+3*J);
   steadyc = Xsol(7+3*J);
        
   % Set up and shock the dynamical system 
   nvar = size(ylist); nvar = nvar(1,1);   
   S = zeros(numstates,history);
   
   if (shocktype == 'g')
      S(numstates - 1,1) = shockway; 
      shocktype = 'spending';
      is = 1;
   elseif (shocktype == 'z')
      S(numstates,1) = shockway;
      shocktype = 'moneydemand';
      is = 0;
   else
      S(numstates - 2,1) = shockway;
      shocktype = 'productivity';
      is = 2;
   end
   
   F(:,1) = PI*S(:,1);
   
   % record impulses
   for time = 2:1:history
      S(:,time) = M*S(:,time - 1);
      F(:,time) = PI*S(:,time);
	end      
   
   % Locate variables of interest   
   ic = find(ylist(:,1) == 'c');
   iR = find(ylist(:,1) == 'R'); iR = iR(1);
   iShare = find(ylist(:,1) == 's'); iShare = iShare(1:J);
   in = find(ylist(:,1) == 'n');
   ii = find(ylist(:,1) == 'i');
   ixi = find(ylist(:,1) == 'x');
   ilambda = 9 + 3*J;
   ileisure = 10 + 3*J;
   
   % convert inverse inflation into inflation
   inflation = F(ii,:);
   inflation = -1*inflation;
   
   consumption = F(ic,:);
   labour = F(in,:);
   nominalrate = F(iR,:);
   Shares = F(iShare,:);
   xi = F(ixi, :);
   leisure = F(ileisure, :);
   lambda = F(ilambda, :);
   wage = -leisure - lambda;
   markup = S(numstates - 2,:) - wage;
   
   % generate price level deviations from trend
   zp = 1;				% zero index
   P(zp) = 1;
   Ptrend(zp) = 1;
   Pbartrend(zp) = 1 + steadynominalrate;
   
   for i = 1:1:history
      nominalratelevel(i) = nominalrate(i) + steadynominalrate;
      infllevel(i) = inflation(i)*steadyinfl + steadyinfl;
      P(zp+ i) = P(zp + i-1)*infllevel(i);
      Pbar(i) = P(i)*(1 + nominalratelevel(i));
      Ptrend(zp + i) = Ptrend(zp + i - 1)*steadyinfl;
      Pbartrend(zp+i) = Pbartrend(zp + i - 1)*steadyinfl;
      xilev(i) = xi(i)*steadyxi + steadyxi;
   end
    
   Pbarinfllevel = (Pbar(2:history) - Pbar(1:history-1))./Pbar(1:history-1);
   P = P(2:history+1);
   Ptrend = Ptrend(2:history + 1);
   Pbartrend = Pbartrend(1:history);
   Mtrend = Ptrend*(1 - steadyxi)*steadyc;
   Pdev = (P - Ptrend)./Ptrend;
   Pbardev = (Pbar - Pbartrend)./Pbartrend;
   M = (1 - xilev).*consumption.*P;
   Mdev = (M - Mtrend)./Mtrend;
    
   infldev = infllevel - steadyinfl;   
   realrate = nominalrate - infldev; 
   realratelevel = realrate + steadyrealrate;
   
   nominalrate = nominalrate*4;
   realrate = realrate*4;
   nominalratelevel = nominalratelevel*4;
   realratelevel = realratelevel*4;
   infllevel = (infllevel - 1)*4;
   Pbarinfllevel = (Pbarinfllevel)*4;
   
   % plot figures
   time = [1:1:history];
   
   figure
   
   subplot(311)
   
   title ([ shocktype ' shock process ' ] )
   hold on
   plot(time, S(numstates - is,:), 'm-')
      
   subplot (312)
   
   title ([ ' annualised inflation rate level (LR = ', num2str((steadyinfl-1)*4),'| FR = ', num2str(-steadyrealrate*4),') ' ] )
   hold on
   plot(time, infllevel, 'r')
   
   subplot (313)
   
   colours = 'mcrgbky';
   
   title ( ' shares ')
   hold on
   
   for i = 1:1:J
      colour = i;
      colour = min(i, mod(i,length(colours))+ 1);
      colourchoice = colours(colour);
      plot(time, Shares(i,:), colourchoice)
      [yval, xval] = max(Shares(i,:));
      labelnow = ['s(' num2str(i) ')'];
      text(xval - 0.25, yval + 0.0025, labelnow)
      hold on
   end
   
   figure
   
   subplot(221)
   
   title( ' consumption [r--] and labour [b-] ' )
   hold on
   plot( time, consumption, 'r--', time, labour, 'b')
     
   subplot(222)
   
   title ( ' \xi: share of credit goods in levels ' )
   hold on
   plot(time, xilev)  
      
   subplot(223)
   
   title( ' annualised real (--)/ nominal rate deviations ' )
   hold on
   plot(time, nominalratelevel, 'b-', time, realratelevel, 'r--')
     
   subplot(224)
   
   title ( ' Prices % dev. trend ')
   hold on
   plot(time, Pdev)
   
   figure
   
   subplot(221)   
   title ( ' annualised real (--)/ nominal rate deviations ' )
   hold on
   plot (time, realrate, 'r--', time, nominalrate, 'b-')
   
   subplot(222)
   title ( ' Money % dev. trend ' )
   hold on
   plot(time, Mdev)
   
   subplot(223)
   title ( ' wage rate ' )
   hold on
   plot(time, wage)
   
   subplot(224)
   title ( ' markup ' )
   hold on
   plot(time, markup)
   
   figure
   
   title ( ' Pbar and P [--] % dev. trend ')
   hold on
   plot(time, Pbardev, 'b-', time, Pdev, 'r--')
   
   figure
   
   title ( ' Pbar and P [--] annualised inflation rates ')
   hold on
   plot(time(1:history-1), infllevel(1:history-1), 'r--', time(1:history-1), Pbarinfllevel(1:history-1), 'b-')

end