function [F] = system0(X,Y, shock, param);

% Aubhik 23.05.00 - 05.06.00, 14.11.2000 extended generalised beta distribution for credit costs on 29.08.2001
%
% System of Equations program
%
% Adapted for the new cost of credit formulation using the Beta Distribution.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%														%
%	King Khan and Wolman Optimal Monetary Policy Model	%
%														%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%
%	Parameter Setup  %
%%%%%%%%%%%%%%%%%%%%%%

Abar = shock(1);					% multiplicative productivity 
xbar = shock(2);					% additive productivity
zbar = shock(3);					% money demand disturbance

epsilon = param(1);     			% elasticity of substitution across consumption goods
theta = param(2);					% preference weight on leisure
eta = param(3);         			% elasticity of labour supply
beta = param(4);					% subjective discount factor
bplanner = param(5);
p0cc = param(6);					% cost of credit fraction of goods without credit cost parameter
p1cc = param(7);
alphacc = param(8);         		% cost of credit beta distribution parameter
betacc = param(9);              	% cost of credit beta distrbution parameter	
kcc = param(10);					% cost of credit scale parameter
J = param(11);						% nominal price setting horizon
omega = param(12: 11 + J);          % adjustment probabilities for price-setting			

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           %
%	Variable Definition     %
%                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% multipliers and variables 

% static multipliers at time t
mdelta = X(1);
mxi = X(2);
momega = X(3);
mrho = X(4);
mlambda = X(5);
mq = X(6:4+J);						% J - 1 evolution of shares multipliers q(j), j = 1,...,J-1.

% choices at time t
mphi = X(5+J);		 				% commitment multiplier for price setting chosen today
V = X(6+J:4+2*J);					% perhaps co-states, J-1 aux. vars. for price-setting, Vj, j = 1,...,J-1.
mpsi = X(5+2*J);					% commitment multiplier for the household's euler chosen today
s = X(6+2*J:5+3*J);                 % J shares for firms, s(0,t),...,s(J-1,t). 
R = X(6+3*J);						% nominal rate of interest between this period and the next
c = X(7+3*J);						% consumption
xi = X(8+3*J);						% fraction of goods bought using credit
lambda = X(9+3*J);                  % relative price of output for household
l = X(10+3*J);						% leisure
n = X(11+3*J);						% hours worked
iinfl = X(12+3*J);                  % inverse of inflation factor

% state variables at the beginning of period t
slag = X(13+3*J: 11+4*J);           % J - 1 lagged shares s(0,t-1),...s(J-2,t-1).
mphilag = X(12+4*J:10+5*J);         % the J-1 lagged commitment multipliers phi(t-1),...,phi(t-(J-1))
Rlag = X(11+5*J);					% lagged nominal rate
mpsilag = X(12+5*J);				% lagged commitment multipliers

% terms from time t+1
mqlead = Y(6:4+J);
Vlead = Y(6+J:4+2*J);
iinfllead = Y(12+3*J);
lambdalead = Y(9+3*J);
slead = Y(13+3*J: 11+4*J);
mphilead = Y(12 + 4*J:10 + 5*J);
Rlead = Y(11 + 5*J);
mpsilead = Y(12 + 5*J);

a = Abar + zbar;					% fake inclusion of money demand shock
x = xbar;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%																		%
%	Constraints, First Order Conditions and Additional Laws of Motion	%
%																		%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% numerical terms
epsilon0 = (epsilon - 1)/epsilon;
epsilon1 = -1/epsilon;
[U, D1U, D2U, D11U, D22U, D12U] = Uterms(c,l, theta, eta);
D21U = D12U;

% the argument for the beta inverse cost distribution function. Note the adjustment by kcc

%xbeta = R*c*lambda/(D2U*kcc); 
xbeta = R*c*lambda/(D2U);

% derivatives of the xbeta function with respect to its arguments
DRxbeta = xbeta/R;
Dlambdaxbeta = xbeta/lambda;

%Dcxbeta = xbeta/c - xbeta*(D21U/(kcc*D2U));
%Dlxbeta = -xbeta*(D22U/(kcc*D2U));
Dcxbeta = xbeta/c - xbeta*(D21U/(D2U));
Dlxbeta = -xbeta*(D22U/(D2U));

[IntFbeta, Fbeta, DFbeta] = Fterms(xbeta/kcc, alphacc, betacc, kcc, p0cc, p1cc);

D21U = D12U;

%%%%%%%%%%%%%%%%%%%%%%	
%	the constraints	 %
%%%%%%%%%%%%%%%%%%%%%%

% household FOCs for consumption and share of goods bought with credit, 
% constraints on time, production and shares, equations (12) - (16)
F(1) = -lambda*(1 + R*(1 - xi)) + D1U;
F(2) = -xi + Fbeta;								% these are the 2 FOCs 
F(3) = 1 - l - n - IntFbeta;					% affected by the beta dist.
F(4) = a*n - sum(omega.*s)*(c+x);
F(5) = sum(omega.*(s.^epsilon0)) - 1;

% J - 1 share laws of motion equations (17).
zp = 1;	% zero place holder
place = 5;
for i = 1:1:J-1
   place = place + 1;
   F(place) =  (s(zp + i))^epsilon1 -iinfl*((slag(zp + i - 1))^epsilon1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%	the optimal policy first order conditions %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% equation 18: J-1 period price-setting equation
place = place + 1;

Xv(zp) = Xterm(lambda, s(zp) , c+x, l, a, epsilon, D2U);
F(place) = omega(zp)*Xv(zp) + beta*Vlead(1);
if (J > 3)
for i = 1:1: J - 2
   place = place + 1;
   Xv(zp + i) = Xterm(lambda, s(zp + i) , c+x, l, a, epsilon, D2U);
   F(place) = omega(zp + i )*Xv(zp + i) + beta*Vlead(i+1) - V(i);
end
end

place = place + 1;
Xv(zp + J - 1) = Xterm(lambda, s(zp + J-1) , c+x, l, a, epsilon, D2U);
F(place) = omega(zp + J-1)*Xv(zp + J-1) - V(J-1);

% equation 19: household's euler equation
place = place + 1;
F(place) = lambda - beta*((1 + R)*iinfllead)*lambdalead;

% equations 20-22: share 0, share j and share J-1 FOCs
place = place + 1;
D2Xv(zp) = D2X(lambda, s(zp), c+x, l, a, epsilon, D2U);
snow = s(zp);
F(place) = mphi*omega(zp)*D2Xv(zp) + (beta/epsilon)*iinfl*mqlead(1)*(snow^(epsilon1 -1 )) ...
   - mrho*omega(zp)*(c + x) + mlambda*omega(zp)*(snow^epsilon1);

if (J > 3)
for i = 1:1:J-2
   D2Xv(zp+i) = D2X(lambda, s(zp+i), c+x, l, a, epsilon, D2U);
   place = place + 1;
   snow = s(zp+i);
   F(place) = mphilag(i)*omega(zp+i)*D2Xv(zp + i) + (beta/epsilon)*mqlead(i+1)*iinfl*(snow^(epsilon1 - 1)) ...
      - mrho*omega(zp+i)*(c + x) + mlambda*omega(zp + i)*(snow^epsilon1) + epsilon1*mq(i)*(snow^(epsilon1 - 1));
end
end

place = place + 1;
snow = s(zp + J-1);
D2Xv(zp+ J - 1) = D2X(lambda, snow, c+x, l, a, epsilon, D2U);
F(place) = mphilag(J-1)*omega(zp + J-1)*D2Xv(zp + J-1) - mrho*omega(zp + J-1)*(c + x) ...
   + mlambda*omega(zp + J - 1)*(snow^epsilon1) + epsilon1*mq(J-1)*(snow^(epsilon1-1));

% equation 23: FOC for interest rate. FINAL TERM ADDED FOR beta
place = place + 1;
F(place) = -mdelta*lambda*(1 - xi) + mxi*DFbeta*DRxbeta - beta*mpsi*(lambdalead*iinfllead);
F(place) = F(place) - momega*xbeta*DFbeta*DRxbeta;	

% equations 24-29: FOCs for consumption, xi, lambda, l, n and PI.
% intermediate terms
term0c = 0;	
term0lambda = 0;
term0l = 0;
term1c = 0;
mphiterms = [mphi mphilag];

for i = 0:1:J-1
   % IT IS UNCLEAR WHETHER DISCOUNT = 1 OR BETA^i
   discount = 1;
   D3Xv(zp+i) = D3X(lambda, s(zp + i), c+x, l, a, epsilon, D2U, D21U);
   D1Xv(zp+i) = D1X(lambda, s(zp + i), c+x, l, a, epsilon, D2U);
   D4Xv(zp+i) = D4X(lambda, s(zp + i), c+x, l, a, epsilon, D22U);   
   term0c = term0c + mphiterms(zp+i)*discount*omega(zp+i)*D3Xv(zp + i);
   term0lambda = term0lambda + mphiterms(zp+i)*discount*omega(zp+i)*D1Xv(zp + i);
   term0l = term0l + mphiterms(zp+i)*discount*omega(zp+i)*D4Xv(zp + i);
   term1c = term1c + omega(zp+i)*s(zp+i);
end

place = place + 1;	% consumption FINAL TERM ADDED FOR beta
F(place) = D1U + term0c + mdelta*D11U + mxi*DFbeta*Dcxbeta - mrho*term1c;
F(place) = F(place) - momega*xbeta*DFbeta*Dcxbeta;

place = place + 1;	% xi
F(place) = mdelta*lambda*R - mxi;

place = place + 1;

% lambda FINAL TERM ADDED FOR beta
F(place) = term0lambda + (mpsi - mpsilag*(1 + Rlag)*iinfl) - mdelta*(1 + R*(1 - xi)) + mxi*DFbeta*Dlambdaxbeta;
F(place) = F(place) - momega*xbeta*DFbeta*Dlambdaxbeta;

place = place + 1; % l FINAL TERM ADDED FOR beta
F(place) = D2U + term0l + mdelta*D12U - mxi*DFbeta*Dlxbeta - momega;
F(place) = F(place) - momega*xbeta*DFbeta*Dlxbeta;

place = place + 1;	% n
F(place) = -momega + mrho*a;

place = place + 1;	% iinfl
term1pi = 0;
for i = 1:1:J-1
   term1pi = term1pi + mq(i)*(slag(i)^epsilon1);
end
F(place) = -mpsilag*lambda*(1+Rlag) - term1pi;

% - CHANGED TO + IN FIRST TERM mpsilag ABOVE

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Additional constraints: laws of motion for the state vector	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 0:1:J-2
   place = place + 1;
   F(place) = slead(zp + i) - s(zp+i);
end

place = place + 1;
F(place) = mphilead(zp) - mphi;

if (J > 3)
for i = 1:1:J-2
   place = place + 1;
   F(place) = mphilead(zp + i) - mphilag(i);
end
end

place = place + 1;
F(place) = Rlead - R;
place = place + 1;
F(place) = mpsilead - mpsi;

%%%%%%%%%%%%%%%%%%%%%
%					%
%	Subroutines		%
%					%
%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%   
%				%
%	Utility     %
%				%
%%%%%%%%%%%%%%%%%

function [U, D1U, D2U, D11U, D22U, D12U] = Uterms(c,l, theta, eta);

etaterm = 1 - eta;
U    = log(c) + theta*(l^(1-eta))/(1-eta);
D1U  = 1/c;													
D2U  = theta*(l^-eta);						
D11U = -1*D1U*D1U;										
D22U = -eta*theta*(l^-(1+eta));	
D12U = 0;															

%%%%%%%%%%%%%
%			%
%	Xterms	%
%			%
%%%%%%%%%%%%%

function [X] = Xterm(lambda, share, c, l, a, epsilon, D2U);

epsilonterm = (epsilon - 1)/epsilon;

X = share^epsilonterm;
X = (1-epsilon)*X + epsilon*share*(D2U/(lambda*a));
X = lambda*X*c;

function [X] = D1X(lambda, share, c, l, a, epsilon, D2U);

epsilonterm = (epsilon - 1)/epsilon;

X = share^epsilonterm;
X = (1-epsilon)*X; 
X = X*c;

function [X] = D2X(lambda, share, c, l, a, epsilon, D2U);
% derivative of X wrt share
epsilonterm0 = (epsilon - 1)/epsilon;
epsilonterm1 = -1/epsilon;

X = epsilonterm0*(1-epsilon)*(share^epsilonterm1);
X = X + epsilon*(D2U/(lambda*a));
X = lambda*X*c;

function [X] = D3X(lambda, share, c, l, a, epsilon, D2U, D21U);

epsilonterm = (epsilon - 1)/epsilon;

X = share^epsilonterm;
X = (1-epsilon)*X + epsilon*share*(D2U/(lambda*a));
X = lambda*X;

X = X + c*epsilon*share*(D21U/a);

function [X] = D4X(lambda, share, c, l, a, epsilon, D22U);

epsilonterm = (epsilon - 1)/epsilon;

X = epsilon*share*(D22U/a);
X = X*c;

function [X] = D5X(lambda, share, c, l , a , epsilon, D2U);

epsilonterm = (epsilon - 1)/epsilon;

X = epsilon*share*D2U*(-1/a^2);
X = X*c;

%%%%%%%%%%%%%
%			%
%	Credit	%
%			%
%%%%%%%%%%%%%

function [IntF, F, DF] = Fterms(xadj, alphacc, betacc, kcc, p0cc, p1cc);

B1 = beta(alphacc, betacc);
B2 = beta(alphacc + 1, betacc);

if (xadj < 1)
   if (xadj > 0)
      B3 = betainc(xadj, alphacc, betacc);
      B4 = betainc(xadj, alphacc + 1, betacc);
   else
      B3 = 0;
      B4 = 0;
   end
else
   B3 = 1;
   B4 = 1;
end

F = p0cc + p1cc*B3;
IntF = kcc*p1cc*(B2/B1)*B4;
DF = p1cc*(1/kcc)*(xadj^(alphacc - 1))*( (1 - xadj)^(betacc-1))/B1;


