function [F] = system0CE(X,Y, shock, param, iinfl);

% COMPETITIVE EQUILIBRIUM PROGRAMS INVOLVE SETTING THE INFLATION RATE.
% 04.09.01

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%														%
%  King Khan and Wolman COMPETITIVE EQUILIBIUM MODEL	%
%														%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%
%	Parameter Setup	 %
%%%%%%%%%%%%%%%%%%%%%%

Abar = shock(1);					% multiplicative productivity 
xbar = shock(2);					% additive productivity
zbar = shock(3);					% money demand disturbance

epsilon = param(1);                 % elasticity of substitution across consumption goods
theta = param(2);					% preference weight on leisure
eta = param(3);                     % elasticity of labour supply
beta = param(4);					% subjective discount factor
bplanner = param(5);
p0cc = param(6);					% cost of credit fraction of goods without credit cost parameter
p1cc = param(7);
alphacc = param(8);                 % cost of credit beta distribution parameter
betacc = param(9);                  % cost of credit beta distrbution parameter	
kcc = param(10);					% cost of credit scale parameter
J = param(11);						% nominal price setting horizon
omega = param(12: 11 + J);          % adjustment probabilities for price-setting			
iinfl = param(12 + J);              % inverse of inflation factor
weights = param(13+J:14+J);

if (weights(1) ~= -999)
   % endongeous determination of inflation rate, not available for steady state computation
   endo = 'y';
   iinfl0 = iinfl;
   iinfl = X(6+2*J);
   iinfllead = Y(6+2*J);
   R0 = param(15+J); 
   c0 = param(16+J);
else 
   % exogenous inflation rate as in the steady state
   endo = 'n';
   iinfllead = iinfl;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       	%
%	Variable Definition     %
%                       	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% multipliers and variables 

% static multipliers at time t

% choices at time t
V = X(1:J-1);						% perhaps co-states, J-1 aux. vars. for price-setting, Vj, j = 1,...,J-1.
s = X(J:2*J -1);					% J shares for firms, s(0,t),...,s(J-1,t). 
R = X(2*J);							% nominal rate of interest between this period and the next
c = X(1+2*J);						% consumption
xi = X(2+2*J);						% fraction of goods bought using credit
lambda = X(3+2*J);              	% relative price of output for household
l = X(4+2*J);						% leisure
n = X(5+2*J);						% hours worked

% state variables at the beginning of period t
slag = X(6+2*J: 4+3*J);             % J - 1 lagged shares s(0,t-1),...s(J-2,t-1).
Rlag = X(5+3*J);					% lagged nominal rate

% terms from time t+1
Vlead = Y(1:J-1);
%iinfllead = Y(6+2*J);
lambdalead = Y(3+2*J);
slead = Y(6+2*J: 4+3*J);
Rlead = Y(5+3*J);

a = Abar + zbar;					% fake inclusion of money demand shock
x = xbar;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%																		%
%	Constraints, First Order Conditions and Additional Laws of Motion	%
%																		%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% numerical terms
epsilon0 = (epsilon - 1)/epsilon;
epsilon1 = -1/epsilon;
[U, D1U, D2U, D11U, D22U, D12U] = Uterms(c,l, theta, eta);
D21U = D12U;

% the argument for the beta inverse cost distribution function. Note the adjustment by kcc

%xbeta = R*c*lambda/(D2U*kcc); 
xbeta = R*c*lambda/(D2U);

% derivatives of the xbeta function with respect to its arguments
DRxbeta = xbeta/R;
Dlambdaxbeta = xbeta/lambda;

%Dcxbeta = xbeta/c - xbeta*(D21U/(kcc*D2U));
%Dlxbeta = -xbeta*(D22U/(kcc*D2U));
Dcxbeta = xbeta/c - xbeta*(D21U/(D2U));
Dlxbeta = -xbeta*(D22U/(D2U));

[IntFbeta, Fbeta, DFbeta] = Fterms(xbeta/kcc, alphacc, betacc, kcc, p0cc, p1cc);

D21U = D12U;

%%%%%%%%%%%%%%%%%%%%%%	
%	the constraints	 %
%%%%%%%%%%%%%%%%%%%%%%

% household FOCs for consumption and share of goods bought with credit, 
% constraints on time, production and shares, equations (12) - (16)
F(1) = -lambda*(1 + R*(1 - xi)) + D1U;
F(2) = -xi + Fbeta;								% these are the 2 FOCs 
F(3) = 1 - l - n - IntFbeta;					% affected by the beta dist.
F(4) = a*n - sum(omega.*s)*(c+x);
F(5) = sum(omega.*(s.^epsilon0)) - 1;

% J - 1 share laws of motion equations (17).
zp = 1;	% zero place holder
place = 5;
for i = 1:1:J-1
   place = place + 1;
   F(place) =  (s(zp + i))^epsilon1 -iinfl*((slag(zp + i - 1))^epsilon1);
end

% equation 18: J-1 period price-setting equation
place = place + 1;

Xv(zp) = Xterm(lambda, s(zp) , c+x, l, a, epsilon, D2U);
F(place) = omega(zp)*Xv(zp) + beta*Vlead(1);
if (J > 3)
for i = 1:1: J - 2
   place = place + 1;
   Xv(zp + i) = Xterm(lambda, s(zp + i) , c+x, l, a, epsilon, D2U);
   F(place) = omega(zp + i )*Xv(zp + i) + beta*Vlead(i+1) - V(i);
end
end

place = place + 1;
Xv(zp + J - 1) = Xterm(lambda, s(zp + J-1) , c+x, l, a, epsilon, D2U);
F(place) = omega(zp + J-1)*Xv(zp + J-1) - V(J-1);

% equation 19: household's euler equation
place = place + 1;
F(place) = lambda - beta*((1 + R)*iinfllead)*lambdalead;

if (endo == 'y')
	% interest rate determination rule which makes inflation rate endogenous
	place = place + 1;
   F(place) = R - R0 - weights(1)*(c-c0) - weights(2)*(1/iinfl - 1/iinfl0);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Additional constraints: laws of motion for the state vector	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 0:1:J-2
   place = place + 1;
   F(place) = slead(zp + i) - s(zp+i);
end

place = place + 1;
F(place) = Rlead - R;


%%%%%%%%%%%%%%%%%%%%%
%                   %
%	Subroutines		%
%					%
%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%
%				%
%	Utility     %
%				%
%%%%%%%%%%%%%%%%%

function [U, D1U, D2U, D11U, D22U, D12U] = Uterms(c,l, theta, eta);

etaterm = 1 - eta;
U    = log(c) + theta*(l^(1-eta))/(1-eta);
D1U  = 1/c;													
D2U  = theta*(l^-eta);						
D11U = -1*D1U*D1U;										
D22U = -eta*theta*(l^-(1+eta));	
D12U = 0;															

%%%%%%%%%%%%%%%%%
%				%
%	 Xterms     %
%				%
%%%%%%%%%%%%%%%%%

function [X] = Xterm(lambda, share, c, l, a, epsilon, D2U);

epsilonterm = (epsilon - 1)/epsilon;

X = share^epsilonterm;
X = (1-epsilon)*X + epsilon*share*(D2U/(lambda*a));
X = lambda*X*c;

%%%%%%%%%%%%%
%			%
%	Credit	%
%			%
%%%%%%%%%%%%%

function [IntF, F, DF] = Fterms(xadj, alphacc, betacc, kcc, p0cc, p1cc);

B1 = beta(alphacc, betacc);
B2 = beta(alphacc + 1, betacc);

if (xadj < 1)
   if (xadj > 0)
      B3 = betainc(xadj, alphacc, betacc);
      B4 = betainc(xadj, alphacc + 1, betacc);
   else
      B3 = 0;
      B4 = 0;
   end
else
   B3 = 1;
   B4 = 1;
end

F = p0cc + p1cc*B3;
IntF = kcc*p1cc*(B2/B1)*B4;
DF = p1cc*(1/kcc)*(xadj^(alphacc - 1))*( (1 - xadj)^(betacc-1))/B1;


