function [X1, F1, refinement] = steadyneq(functionfile, XGuess, shock, param, lambda, tolerance, flag)

% Adapted 03062000 from neq: non-linear equation solver 
% Aubhik Khan 02061999 revised 05061999
% 
% function [X1, F1, refinement] = SteadyNEQ('filename', XGuess, shock, param, lambda, tolerance, flag)
%
% required inputs:
%	filename    (must be in quotes) function containing system of nonlinear equations 							 
% 	XGuess      initial conjecture of the root of this system
%   shock       mean values for exogenous stochastic processes
%   param       parameters of the model
%
% optional inputs: 
%   lambda      smoothing parameter for Newton-Rhapson algorithm
%	tolerance   paramter to determine approximate convergence of estimates of the 
%              root of the system.
%	flag        output display indicator flag = 1 displays iterations flag = 2 
%              also displays title and result
%
% output:
%	X1          the root of the system of equations
% 	F1          the error at this solution 
% 	refinment   the distance from the initial guess

% prepare display
flag1 = 0; flag2 = 0;

if (nargin < 4)		% cannot operate without initial guess
   disp ( ' there are limits to what you might expect ' )
   break 
elseif (nargin < 5)	% implement default tolerance if omitted
   lambda = 1;
elseif (nargin < 6)
   tolerance = 1e-10;
elseif (flag == 1)	% implement desired display choices
   flag1 = 1;
elseif (flag == 2)
   flag1 = 1; flag2 = 1;
end

% determine size of system of equations
eqsize = length(XGuess);

% prepare for numerical derivatives
h = tolerance; hvec = ones(1,eqsize)*h; H = diag(hvec,0);

% record iterations starting with conjecture
Xplace(1,:) = XGuess;
place = 1; precisionx = place;

% set up intial function evaluation in accordance with variable 
% renaming conventions used in iterative while loop below
X1 = XGuess;
F1 = feval(functionfile, X1, shock, param);

% iterate starting with conjecture to locate root of system
while precisionx > tolerance
 	
	X0 = X1; F0 = F1;                               % update current state of system 
   J = zeros(eqsize,eqsize);                        % reset Jacobian matrix dimension
      
   % compute numerical derivatives prepared to determine Jacobian
   % the ij-th term contains the derivative of Fi wrt xj
   for i = 1:1:eqsize
      jacobianentry = (feval(functionfile, X0 + H(i,:), shock, param) - F0)./h;
      J(:,i) = jacobianentry';
   end
      
   % implement the Newton step: choose next point in the argument space
	NewtonStep = J\F0';                             % NewtonStep = inv(J)*F'; % equivalent alternatives
   X1 = X0 - lambda.*NewtonStep';                   % X1 is new estimate of root given X0 and J(X0)
   F1 = feval(functionfile, X1, shock, param);		% update system evaluation for new estimate
   place = place + 1;
   X(place,:) = X1;                                 % record the new estimate
   precisionx = sum(abs(X1 - X0));                  % metric to determine distance between estimates
      										
   if (flag1 == 1)                                  % if requested display iteration
      disp ( [ num2str(place - 1 ),')  dx = ', num2str(precisionx), '   f = ', num2str(F1*F1'), ' max X = ', num2str(max(abs(X1)))] );
   end
        
end

refinement = X1 - XGuess;

if (flag2 == 1)
   disp ( ' ' )
   disp (  '     Newton solution method yields ' )
   disp ( ['         X = ',num2str(X1) ] )
   disp ( ['     error = ', num2str(sqrt(F1*F1')) ] )
   disp ( [' refinment = ', num2str(refinement) ] )
	disp ( ' ' )
end 
   
  