% Khan, King and Wolman (2000) Steady State solution driver program
%
% Aubhik Khan 26.05.2000, 02.06.00, 14.11.2000 adjusted for use of beta distribution
% 29.08.2001 generalised for Bob's extended generalised beta distribution with p1cc.
% 30.08.2001 adapted to allow for a homotopic approach, see august and august2.
%
% computes steady state using the parameter values and initial condition delcared below.  The 
% result is stored within a mat-file which is give a name which the user inputs.  
%
% related programs:
% 
% SteadyState		sets Y(t+1)=Y(t)=Xsol and X(t) = X* and evaluates SYSTEM2
% SYSTEM2			contains a system of first-order conditions F(Y(t+1),Y(t),X(t)) = 0
% SteadyNEQ			Basic non-linear equation solver
%
% Please note that Y, below, is the future variables, X is the current variables and shock are 
% the exogenous variables.  
%
% set solve ~= 'y' if you wish to use a solution as the initial condition.

clear
clc

homotopy = 'y';                     % record the solution Xsol to use as an initial condition
solve = 'y';						% resolve the steady state?

% Declare variable locations and X and param within a mat-file

inputfile = input( ' parameter filename: ','s');

cd Calibrations
eval (inputfile);
cd ..

% If a previous solution saved Xsol in a file named guess.m, and if 
% the current inputfile loads this initial condition, then it will be 
% used instead of direct term by term entry in the calibration file.   

if (exist('Xsol') ~= 1)
    
    disp ( ' using directly entered initial conditions ' )
    
    % static multipliers at time t
    X(1) = mdelta;
    X(2) = mxi;
    X(3) = momega;
    X(4) = mrho;
    X(5) = mlambda;
    X(6:4+J) = mqvec;
    
    s = svec;
    slag = s(1:J-1);
    mphilag = mphi*ones(1,J-1);
    Rlag = R;
    mpsilag = mpsi;
    
    % choices at time t
    X(5+J) = mphi;
    X(5+2*J) = mpsi;					
    X(6+2*J:5+3*J) = s; 
    X(6+3*J) = R;						% nominal rate of interest between this period and the next
    X(7+3*J) = c;						% consumption
    X(8+3*J) = xi;						% fraction of goods bought using credit
    X(9+3*J) = lambda;                  % relative price of output for household
    X(10+3*J) = l;						% leisure
    X(11+3*J) = n;						% hours worked
    X(12+3*J) = iinfl;                  % inflation
    
    % state variables at the beginning of period t
    X(13+3*J: 11+4*J) = slag;
    X(12+4*J:10+5*J) = mphilag;
    X(11+5*J) = Rlag;					
    X(12+5*J) = mpsilag;				
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    %						%
    %	Computation of V(j)	%
    %						%
    %%%%%%%%%%%%%%%%%%%%%%%%%
    
    epsilonterm = (epsilon - 1)/epsilon;
    zp = 1;
    D2U = theta*(l^-eta);
    V = zeros(1,J);
    
    for i = J-1:-1:1
        share = s(zp + i);
        Xv = share^epsilonterm;
        Xv = (1-epsilon)*Xv + epsilon*share*(D2U/(lambda*Abar));
        Xv = lambda*Xv*c;
        V(i) = omega(zp+i)*Xv + beta*V(i+1);
    end
    
    V = V(1:J-1);
    
    X(6+J:4+2*J) = V;	
    
else
    
   disp ( ' using Xsol vector from a file named guess ' )
    
    X = Xsol;
    s = X(6+2*J:5+3*J);
    l = X(10+3*J);	
    lambda = X(9+3*J);
    c = X(7+3*J);
    epsilonterm = (epsilon - 1)/epsilon;
    zp = 1;
    D2U = theta*(l^-eta);
    V = zeros(1,J);
    
    for i = J-1:-1:1
        share = s(zp + i);
        Xv = share^epsilonterm;
        Xv = (1-epsilon)*Xv + epsilon*share*(D2U/(lambda*Abar));
        Xv = lambda*Xv*c;
        V(i) = omega(zp+i)*Xv + beta*V(i+1);
    end
    
end

% consistency check for recursive first-order form
share = s(zp);
Xv = share^epsilonterm;
Xv = (1-epsilon)*Xv + epsilon*share*(D2U/(lambda*Abar));
Xv = lambda*Xv*c;
error = omega(zp)*Xv + beta*V(1);

disp ([ ' the recursive first order form implies an error of ', num2str(error) ] )

disp ( ' evaluation of the initial condition ' )
SteadyState(X, shock, param)

if (solve == 'y')
tolerance = 1e-8;

[Xsol, Fsol, refined] = SteadyNEQ('SteadyState', X, shock, param, 1, tolerance, 1);

disp ( ' change in the endogenous variables ' )
disp([X - Xsol])

else
   
   Xsol = X;

end
daytime = datestr(now);

% save results in a mat file that does not overwrite existing files and 
eval ( [ ' cd Calibrations ';]);
filecount = 0;
filexist = exist([savedata '.mat']);
filenameappend = savedata;

while (filexist ~= 0)
   filecount = filecount + 1;
   filenameappend = [savedata '_' num2str(filecount) ];
   filexist = exist([filenameappend '.mat']);
end


eval ( [ ' save ',filenameappend, ' Xsol param shock J daytime' ] );

if (homotopy == 'y')
    disp ( ' saving solution Xsol in file named guess ' )
    save guess Xsol
end

cd ..

disp ( [ ' Parameterisation and Steady State Solution saved as ', filenameappend,'.mat ' ] )

